#ifndef n_RGB565_H
#define n_RGB565_H

/*
 *
 * nucleo/image/encoding/RGB565.H --
 *
 * Copyright (C) Olivier Chapuis
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

namespace nucleo {

void ARGB2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
void RGBA2RGB565(
	unsigned int w, unsigned int h, unsigned char *src,unsigned char *dst);
void RGB2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
void L2RGB565(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);

void RGB5652RGB(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
void RGB5652L(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
void RGB5652ARGB(
	unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
void RGB5652RGBA(
	 unsigned int w, unsigned int h, unsigned char *src, unsigned char *dst);
}

#endif
